@@ -1,14 +1,14 @@
 /*
  * Man page to HTML conversion program.
  *
- * Copyright 2007-2010, 2014 by Apple Inc.
+ * Copyright 2007-2017 by Apple Inc.
  * Copyright 2004-2006 by Easy Software Products.
  *
  * These coded instructions, statements, and computer programs are the
  * property of Apple Inc. and are protected by Federal copyright
  * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
  * which should have been included with this file.  If this file is
- * file is missing or damaged, see the license at "http://www.cups.org/".
+ * missing or damaged, see the license at "http://www.cups.org/".
  */
 
 /*
@@ -1153,8 +1153,11 @@ html_fputs(const char *s,		/* I  - String */
       }
       else
       {
-        if (*s != '\\' && *s == '\"' && *s == '\'' && *s == '-')
+        if (*s != '\\' && *s != '\"' && *s != '\'' && *s != '-')
+        {
           fprintf(stderr, "mantohtml: Unrecognized escape \"\\%c\" ignored.\n", *s);
+          html_putc('\\', fp);
+        }
 
         html_putc(*s++, fp);
       }
