@@ -1,14 +1,14 @@
 //
 // Shared message catalog class for the CUPS PPD Compiler.
 //
-// Copyright 2007-2016 by Apple Inc.
+// Copyright 2007-2017 by Apple Inc.
 // Copyright 2002-2006 by Easy Software Products.
 //
 // These coded instructions, statements, and computer programs are the
 // property of Apple Inc. and are protected by Federal copyright
 // law.  Distribution and use rights are outlined in the file "LICENSE.txt"
 // which should have been included with this file.  If this file is
-// file is missing or damaged, see the license at "http://www.cups.org/".
+// missing or damaged, see the license at "http://www.cups.org/".
 //
 
 //
@@ -58,7 +58,7 @@ ppdcCatalog::ppdcCatalog(const char *l,	// I - Locale
   filename = new ppdcString(f);
   messages = new ppdcArray();
 
-  if (l)
+  if (l && strcmp(l, "en"))
   {
     // Try loading the base messages for this locale...
     char	pofile[1024];		// Message catalog file
